nav_bar_font = ("Arial", 10, "bold")
nav_bar_foreground = "#FFFFFF"
nav_bar_background = "#4F5D75"
nav_bar_button_width = 16

details_font = ("Verdana", 10)
details_background = "#BFC0C0"
details_top_margin = 16
